/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.db2client;

import rx.Observable;
import rx.Single;
import io.vertx.rx.java.RxHelper;
import io.vertx.rx.java.WriteStreamSubscriber;
import io.vertx.rx.java.SingleOnSubscribeAdapter;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Iterator;
import java.util.function.Function;
import java.util.stream.Collectors;
import io.vertx.core.Handler;
import io.vertx.core.AsyncResult;
import io.vertx.core.json.JsonObject;
import io.vertx.core.json.JsonArray;
import io.vertx.lang.rx.RxGen;
import io.vertx.lang.rx.TypeArg;
import io.vertx.lang.rx.MappingIterator;

/**
 * A pool of DB2 connections.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.db2client.DB2Pool original} non RX-ified interface using Vert.x codegen.
 */

@RxGen(io.vertx.db2client.DB2Pool.class)
public class DB2Pool extends io.vertx.rxjava.sqlclient.Pool {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    DB2Pool that = (DB2Pool) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final TypeArg<DB2Pool> __TYPE_ARG = new TypeArg<>(    obj -> new DB2Pool((io.vertx.db2client.DB2Pool) obj),
    DB2Pool::getDelegate
  );

  private final io.vertx.db2client.DB2Pool delegate;
  
  public DB2Pool(io.vertx.db2client.DB2Pool delegate) {
    super(delegate);
    this.delegate = delegate;
  }

  public DB2Pool(Object delegate) {
    super((io.vertx.db2client.DB2Pool)delegate);
    this.delegate = (io.vertx.db2client.DB2Pool)delegate;
  }

  public io.vertx.db2client.DB2Pool getDelegate() {
    return delegate;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#pool} with default options.
   * @param connectionUri 
   * @return 
   */
  public static io.vertx.rxjava.db2client.DB2Pool pool(java.lang.String connectionUri) { 
    io.vertx.rxjava.db2client.DB2Pool ret = io.vertx.rxjava.db2client.DB2Pool.newInstance((io.vertx.db2client.DB2Pool)io.vertx.db2client.DB2Pool.pool(connectionUri));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#pool} with
   * <code>database</code> build from <code>connectionUri</code>.
   * @param connectionUri 
   * @param options 
   * @return 
   */
  public static io.vertx.rxjava.db2client.DB2Pool pool(java.lang.String connectionUri, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.db2client.DB2Pool ret = io.vertx.rxjava.db2client.DB2Pool.newInstance((io.vertx.db2client.DB2Pool)io.vertx.db2client.DB2Pool.pool(connectionUri, options));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#pool} with default options.
   * @param vertx 
   * @param connectionUri 
   * @return 
   */
  public static io.vertx.rxjava.db2client.DB2Pool pool(io.vertx.rxjava.core.Vertx vertx, java.lang.String connectionUri) { 
    io.vertx.rxjava.db2client.DB2Pool ret = io.vertx.rxjava.db2client.DB2Pool.newInstance((io.vertx.db2client.DB2Pool)io.vertx.db2client.DB2Pool.pool(vertx.getDelegate(), connectionUri));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#pool} with
   * <code>database</code> build from <code>connectionUri</code>.
   * @param vertx 
   * @param connectionUri 
   * @param options 
   * @return 
   */
  public static io.vertx.rxjava.db2client.DB2Pool pool(io.vertx.rxjava.core.Vertx vertx, java.lang.String connectionUri, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.db2client.DB2Pool ret = io.vertx.rxjava.db2client.DB2Pool.newInstance((io.vertx.db2client.DB2Pool)io.vertx.db2client.DB2Pool.pool(vertx.getDelegate(), connectionUri, options));
    return ret;
  }

  /**
   * Create a connection pool to the DB2 <code>database</code> configured with the given <code>options</code>.
   * @param database the options for the connection
   * @param options the options for creating the pool
   * @return the connection pool
   */
  public static io.vertx.rxjava.db2client.DB2Pool pool(io.vertx.db2client.DB2ConnectOptions database, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.db2client.DB2Pool ret = io.vertx.rxjava.db2client.DB2Pool.newInstance((io.vertx.db2client.DB2Pool)io.vertx.db2client.DB2Pool.pool(database, options));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#pool} with a specific
   *  instance.
   * @param vertx 
   * @param database 
   * @param options 
   * @return 
   */
  public static io.vertx.rxjava.db2client.DB2Pool pool(io.vertx.rxjava.core.Vertx vertx, io.vertx.db2client.DB2ConnectOptions database, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.db2client.DB2Pool ret = io.vertx.rxjava.db2client.DB2Pool.newInstance((io.vertx.db2client.DB2Pool)io.vertx.db2client.DB2Pool.pool(vertx.getDelegate(), database, options));
    return ret;
  }

  /**
   * Create a connection pool to the DB2 <code>databases</code> with round-robin selection.
   * Round-robin is applied when a new connection is created by the pool.
   * @param databases the list of servers
   * @param options the options for creating the pool
   * @return the connection pool
   */
  public static io.vertx.rxjava.db2client.DB2Pool pool(java.util.List<io.vertx.db2client.DB2ConnectOptions> databases, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.db2client.DB2Pool ret = io.vertx.rxjava.db2client.DB2Pool.newInstance((io.vertx.db2client.DB2Pool)io.vertx.db2client.DB2Pool.pool(databases, options));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#pool} with a specific
   *  instance.
   * @param vertx 
   * @param databases 
   * @param options 
   * @return 
   */
  public static io.vertx.rxjava.db2client.DB2Pool pool(io.vertx.rxjava.core.Vertx vertx, java.util.List<io.vertx.db2client.DB2ConnectOptions> databases, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.db2client.DB2Pool ret = io.vertx.rxjava.db2client.DB2Pool.newInstance((io.vertx.db2client.DB2Pool)io.vertx.db2client.DB2Pool.pool(vertx.getDelegate(), databases, options));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#client} with default options.
   * @param connectionUri 
   * @return 
   */
  public static io.vertx.rxjava.sqlclient.SqlClient client(java.lang.String connectionUri) { 
    io.vertx.rxjava.sqlclient.SqlClient ret = io.vertx.rxjava.sqlclient.SqlClient.newInstance((io.vertx.sqlclient.SqlClient)io.vertx.db2client.DB2Pool.client(connectionUri));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#client} with
   * <code>database</code> build from <code>connectionUri</code>.
   * @param connectionUri 
   * @param options 
   * @return 
   */
  public static io.vertx.rxjava.sqlclient.SqlClient client(java.lang.String connectionUri, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.sqlclient.SqlClient ret = io.vertx.rxjava.sqlclient.SqlClient.newInstance((io.vertx.sqlclient.SqlClient)io.vertx.db2client.DB2Pool.client(connectionUri, options));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#client} with default options.
   * @param vertx 
   * @param connectionUri 
   * @return 
   */
  public static io.vertx.rxjava.sqlclient.SqlClient client(io.vertx.rxjava.core.Vertx vertx, java.lang.String connectionUri) { 
    io.vertx.rxjava.sqlclient.SqlClient ret = io.vertx.rxjava.sqlclient.SqlClient.newInstance((io.vertx.sqlclient.SqlClient)io.vertx.db2client.DB2Pool.client(vertx.getDelegate(), connectionUri));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#client} with
   * <code>database</code> build from <code>connectionUri</code>.
   * @param vertx 
   * @param connectionUri 
   * @param options 
   * @return 
   */
  public static io.vertx.rxjava.sqlclient.SqlClient client(io.vertx.rxjava.core.Vertx vertx, java.lang.String connectionUri, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.sqlclient.SqlClient ret = io.vertx.rxjava.sqlclient.SqlClient.newInstance((io.vertx.sqlclient.SqlClient)io.vertx.db2client.DB2Pool.client(vertx.getDelegate(), connectionUri, options));
    return ret;
  }

  /**
   * Create a pooled client to the DB2 <code>database</code> configured with the given <code>options</code>.
   * @param database the options for the connection
   * @param options the options for creating the pool
   * @return the connection pool
   */
  public static io.vertx.rxjava.sqlclient.SqlClient client(io.vertx.db2client.DB2ConnectOptions database, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.sqlclient.SqlClient ret = io.vertx.rxjava.sqlclient.SqlClient.newInstance((io.vertx.sqlclient.SqlClient)io.vertx.db2client.DB2Pool.client(database, options));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#client} with a specific
   *  instance.
   * @param vertx 
   * @param database 
   * @param options 
   * @return 
   */
  public static io.vertx.rxjava.sqlclient.SqlClient client(io.vertx.rxjava.core.Vertx vertx, io.vertx.db2client.DB2ConnectOptions database, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.sqlclient.SqlClient ret = io.vertx.rxjava.sqlclient.SqlClient.newInstance((io.vertx.sqlclient.SqlClient)io.vertx.db2client.DB2Pool.client(vertx.getDelegate(), database, options));
    return ret;
  }

  /**
   * Create a client backed by a connection pool to the DB2 <code>databases</code> with round-robin selection.
   * Round-robin is applied when a new connection is created by the pool.
   * @param databases the list of servers
   * @param options the options for creating the pool
   * @return the pooled client
   */
  public static io.vertx.rxjava.sqlclient.SqlClient client(java.util.List<io.vertx.db2client.DB2ConnectOptions> databases, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.sqlclient.SqlClient ret = io.vertx.rxjava.sqlclient.SqlClient.newInstance((io.vertx.sqlclient.SqlClient)io.vertx.db2client.DB2Pool.client(databases, options));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.db2client.DB2Pool#client} with a specific
   *  instance.
   * @param vertx 
   * @param databases 
   * @param options 
   * @return 
   */
  public static io.vertx.rxjava.sqlclient.SqlClient client(io.vertx.rxjava.core.Vertx vertx, java.util.List<io.vertx.db2client.DB2ConnectOptions> databases, io.vertx.sqlclient.PoolOptions options) { 
    io.vertx.rxjava.sqlclient.SqlClient ret = io.vertx.rxjava.sqlclient.SqlClient.newInstance((io.vertx.sqlclient.SqlClient)io.vertx.db2client.DB2Pool.client(vertx.getDelegate(), databases, options));
    return ret;
  }

  public io.vertx.rxjava.db2client.DB2Pool connectHandler(io.vertx.core.Handler<io.vertx.rxjava.sqlclient.SqlConnection> handler) { 
    delegate.connectHandler(new Handler<io.vertx.sqlclient.SqlConnection>() {
      public void handle(io.vertx.sqlclient.SqlConnection event) {
        handler.handle(io.vertx.rxjava.sqlclient.SqlConnection.newInstance((io.vertx.sqlclient.SqlConnection)event));
      }
    });
    return this;
  }

  public io.vertx.rxjava.db2client.DB2Pool connectionProvider(java.util.function.Function<io.vertx.rxjava.core.Context,io.vertx.core.Future<io.vertx.rxjava.sqlclient.SqlConnection>> provider) { 
    delegate.connectionProvider(new Function<io.vertx.core.Context,io.vertx.core.Future<io.vertx.sqlclient.SqlConnection>>() {
      public io.vertx.core.Future<io.vertx.sqlclient.SqlConnection> apply(io.vertx.core.Context arg) {
        io.vertx.core.Future<io.vertx.rxjava.sqlclient.SqlConnection> ret = provider.apply(io.vertx.rxjava.core.Context.newInstance((io.vertx.core.Context)arg));
        return ret.map(val -> val.getDelegate());
      }
    });
    return this;
  }

  public static DB2Pool newInstance(io.vertx.db2client.DB2Pool arg) {
    return arg != null ? new DB2Pool(arg) : null;
  }

}
