/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.ext.auth.oauth2.providers;

import rx.Observable;
import rx.Single;
import io.vertx.rx.java.RxHelper;
import io.vertx.rx.java.WriteStreamSubscriber;
import io.vertx.rx.java.SingleOnSubscribeAdapter;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Iterator;
import java.util.function.Function;
import java.util.stream.Collectors;
import io.vertx.core.Handler;
import io.vertx.core.AsyncResult;
import io.vertx.core.json.JsonObject;
import io.vertx.core.json.JsonArray;
import io.vertx.lang.rx.RxGen;
import io.vertx.lang.rx.TypeArg;
import io.vertx.lang.rx.MappingIterator;

/**
 * Simplified factory to create an  for CloudFoundry UAA.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.ext.auth.oauth2.providers.CloudFoundryAuth original} non RX-ified interface using Vert.x codegen.
 */

@RxGen(io.vertx.ext.auth.oauth2.providers.CloudFoundryAuth.class)
public class CloudFoundryAuth {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    CloudFoundryAuth that = (CloudFoundryAuth) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final TypeArg<CloudFoundryAuth> __TYPE_ARG = new TypeArg<>(    obj -> new CloudFoundryAuth((io.vertx.ext.auth.oauth2.providers.CloudFoundryAuth) obj),
    CloudFoundryAuth::getDelegate
  );

  private final io.vertx.ext.auth.oauth2.providers.CloudFoundryAuth delegate;
  
  public CloudFoundryAuth(io.vertx.ext.auth.oauth2.providers.CloudFoundryAuth delegate) {
    this.delegate = delegate;
  }

  public CloudFoundryAuth(Object delegate) {
    this.delegate = (io.vertx.ext.auth.oauth2.providers.CloudFoundryAuth)delegate;
  }

  public io.vertx.ext.auth.oauth2.providers.CloudFoundryAuth getDelegate() {
    return delegate;
  }

  /**
   * Create a OAuth2Auth provider for CloudFoundry UAA
   * @param vertx 
   * @param clientId the client id given to you by CloudFoundry UAA
   * @param clientSecret the client secret given to you by CloudFoundry UAA
   * @param uuaURL the url to your UUA server instance
   * @return 
   */
  public static io.vertx.rxjava.ext.auth.oauth2.OAuth2Auth create(io.vertx.rxjava.core.Vertx vertx, java.lang.String clientId, java.lang.String clientSecret, java.lang.String uuaURL) { 
    io.vertx.rxjava.ext.auth.oauth2.OAuth2Auth ret = io.vertx.rxjava.ext.auth.oauth2.OAuth2Auth.newInstance((io.vertx.ext.auth.oauth2.OAuth2Auth)io.vertx.ext.auth.oauth2.providers.CloudFoundryAuth.create(vertx.getDelegate(), clientId, clientSecret, uuaURL));
    return ret;
  }

  /**
   * Create a OAuth2Auth provider for CloudFoundry UAA
   * @param vertx 
   * @param clientId the client id given to you by CloudFoundry UAA
   * @param clientSecret the client secret given to you by CloudFoundry UAA
   * @param uuaURL the url to your UUA server instance
   * @param httpClientOptions custom http client options
   * @return 
   */
  public static io.vertx.rxjava.ext.auth.oauth2.OAuth2Auth create(io.vertx.rxjava.core.Vertx vertx, java.lang.String clientId, java.lang.String clientSecret, java.lang.String uuaURL, io.vertx.core.http.HttpClientOptions httpClientOptions) { 
    io.vertx.rxjava.ext.auth.oauth2.OAuth2Auth ret = io.vertx.rxjava.ext.auth.oauth2.OAuth2Auth.newInstance((io.vertx.ext.auth.oauth2.OAuth2Auth)io.vertx.ext.auth.oauth2.providers.CloudFoundryAuth.create(vertx.getDelegate(), clientId, clientSecret, uuaURL, httpClientOptions));
    return ret;
  }

  public static CloudFoundryAuth newInstance(io.vertx.ext.auth.oauth2.providers.CloudFoundryAuth arg) {
    return arg != null ? new CloudFoundryAuth(arg) : null;
  }

}
