/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.ext.shell;

import rx.Observable;
import rx.Single;
import io.vertx.rx.java.RxHelper;
import io.vertx.rx.java.WriteStreamSubscriber;
import io.vertx.rx.java.SingleOnSubscribeAdapter;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Iterator;
import java.util.function.Function;
import java.util.stream.Collectors;
import io.vertx.core.Handler;
import io.vertx.core.AsyncResult;
import io.vertx.core.json.JsonObject;
import io.vertx.core.json.JsonArray;
import io.vertx.lang.rx.RxGen;
import io.vertx.lang.rx.TypeArg;
import io.vertx.lang.rx.MappingIterator;

/**
 * An interactive session between a consumer and a shell.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.ext.shell.Shell original} non RX-ified interface using Vert.x codegen.
 */

@RxGen(io.vertx.ext.shell.Shell.class)
public class Shell {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    Shell that = (Shell) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final TypeArg<Shell> __TYPE_ARG = new TypeArg<>(    obj -> new Shell((io.vertx.ext.shell.Shell) obj),
    Shell::getDelegate
  );

  private final io.vertx.ext.shell.Shell delegate;
  
  public Shell(io.vertx.ext.shell.Shell delegate) {
    this.delegate = delegate;
  }

  public Shell(Object delegate) {
    this.delegate = (io.vertx.ext.shell.Shell)delegate;
  }

  public io.vertx.ext.shell.Shell getDelegate() {
    return delegate;
  }

  /**
   * Create a job, the created job should then be executed with the {@link io.vertx.rxjava.ext.shell.system.Job#run} method.
   * @param line the command line creating this job
   * @return the created job
   */
  public io.vertx.rxjava.ext.shell.system.Job createJob(java.util.List<io.vertx.rxjava.ext.shell.cli.CliToken> line) { 
    io.vertx.rxjava.ext.shell.system.Job ret = io.vertx.rxjava.ext.shell.system.Job.newInstance((io.vertx.ext.shell.system.Job)delegate.createJob(line.stream().map(elt -> elt.getDelegate()).collect(Collectors.toList())));
    return ret;
  }

  /**
   * See {@link io.vertx.rxjava.ext.shell.Shell#createJob}
   * @param line 
   * @return 
   */
  public io.vertx.rxjava.ext.shell.system.Job createJob(java.lang.String line) { 
    io.vertx.rxjava.ext.shell.system.Job ret = io.vertx.rxjava.ext.shell.system.Job.newInstance((io.vertx.ext.shell.system.Job)delegate.createJob(line));
    return ret;
  }

  /**
   * @return the shell's job controller
   */
  public io.vertx.rxjava.ext.shell.system.JobController jobController() { 
    if (cached_0 != null) {
      return cached_0;
    }
    io.vertx.rxjava.ext.shell.system.JobController ret = io.vertx.rxjava.ext.shell.system.JobController.newInstance((io.vertx.ext.shell.system.JobController)delegate.jobController());
    cached_0 = ret;
    return ret;
  }

  /**
   * @return the current shell session
   */
  public io.vertx.rxjava.ext.shell.session.Session session() { 
    if (cached_1 != null) {
      return cached_1;
    }
    io.vertx.rxjava.ext.shell.session.Session ret = io.vertx.rxjava.ext.shell.session.Session.newInstance((io.vertx.ext.shell.session.Session)delegate.session());
    cached_1 = ret;
    return ret;
  }

  /**
   * Set a new prompt in this session.
   * @param prompt the new prompt will be calculated when it's needed.
   */
  public void setPrompt(java.util.function.Function<io.vertx.rxjava.ext.shell.session.Session,java.lang.String> prompt) { 
    delegate.setPrompt(new Function<io.vertx.ext.shell.session.Session,java.lang.String>() {
      public java.lang.String apply(io.vertx.ext.shell.session.Session arg) {
        java.lang.String ret = prompt.apply(io.vertx.rxjava.ext.shell.session.Session.newInstance((io.vertx.ext.shell.session.Session)arg));
        return ret;
      }
    });
  }

  /**
   * Close the shell.
   */
  public void close() { 
    delegate.close();
  }

  private io.vertx.rxjava.ext.shell.system.JobController cached_0;
  private io.vertx.rxjava.ext.shell.session.Session cached_1;
  public static Shell newInstance(io.vertx.ext.shell.Shell arg) {
    return arg != null ? new Shell(arg) : null;
  }

}
