/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.ext.web.handler;

import rx.Observable;
import rx.Single;
import io.vertx.rx.java.RxHelper;
import io.vertx.rx.java.WriteStreamSubscriber;
import io.vertx.rx.java.SingleOnSubscribeAdapter;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Iterator;
import java.util.function.Function;
import java.util.stream.Collectors;
import io.vertx.core.Handler;
import io.vertx.core.AsyncResult;
import io.vertx.core.json.JsonObject;
import io.vertx.core.json.JsonArray;
import io.vertx.lang.rx.RxGen;
import io.vertx.lang.rx.TypeArg;
import io.vertx.lang.rx.MappingIterator;

/**
 * Base interface for auth handlers.
 * <p>
 * An auth handler allows your application to provide authentication support.
 * <p>
 * An Auth handler may require a {@link io.vertx.rxjava.ext.web.handler.SessionHandler} to be on the routing chain before it.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.ext.web.handler.AuthenticationHandler original} non RX-ified interface using Vert.x codegen.
 */

@RxGen(io.vertx.ext.web.handler.AuthenticationHandler.class)
public interface AuthenticationHandler extends Handler<io.vertx.rxjava.ext.web.RoutingContext> {

  io.vertx.ext.web.handler.AuthenticationHandler getDelegate();

  /**
   * Something has happened, so handle it.
   * @param event the event to handle
   */
  public void handle(io.vertx.rxjava.ext.web.RoutingContext event);

  public static AuthenticationHandler newInstance(io.vertx.ext.web.handler.AuthenticationHandler arg) {
    return arg != null ? new AuthenticationHandlerImpl(arg) : null;
  }

}

class AuthenticationHandlerImpl implements AuthenticationHandler {
  private final io.vertx.ext.web.handler.AuthenticationHandler delegate;
  
  public AuthenticationHandlerImpl(io.vertx.ext.web.handler.AuthenticationHandler delegate) {
    this.delegate = delegate;
  }

  public AuthenticationHandlerImpl(Object delegate) {
    this.delegate = (io.vertx.ext.web.handler.AuthenticationHandler)delegate;
  }

  public io.vertx.ext.web.handler.AuthenticationHandler getDelegate() {
    return delegate;
  }

  /**
   * Something has happened, so handle it.
   * @param event the event to handle
   */
  public void handle(io.vertx.rxjava.ext.web.RoutingContext event) { 
    delegate.handle(event.getDelegate());
  }

}
