/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.ext.web.handler;

import rx.Observable;
import rx.Single;
import io.vertx.rx.java.RxHelper;
import io.vertx.rx.java.WriteStreamSubscriber;
import io.vertx.rx.java.SingleOnSubscribeAdapter;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Iterator;
import java.util.function.Function;
import java.util.stream.Collectors;
import io.vertx.core.Handler;
import io.vertx.core.AsyncResult;
import io.vertx.core.json.JsonObject;
import io.vertx.core.json.JsonArray;
import io.vertx.lang.rx.RxGen;
import io.vertx.lang.rx.TypeArg;
import io.vertx.lang.rx.MappingIterator;

/**
 * An auth handler that provides HTTP Basic Authentication support.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.ext.web.handler.DigestAuthHandler original} non RX-ified interface using Vert.x codegen.
 */

@RxGen(io.vertx.ext.web.handler.DigestAuthHandler.class)
public class DigestAuthHandler implements io.vertx.rxjava.ext.web.handler.AuthenticationHandler, Handler<io.vertx.rxjava.ext.web.RoutingContext> {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    DigestAuthHandler that = (DigestAuthHandler) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final TypeArg<DigestAuthHandler> __TYPE_ARG = new TypeArg<>(    obj -> new DigestAuthHandler((io.vertx.ext.web.handler.DigestAuthHandler) obj),
    DigestAuthHandler::getDelegate
  );

  private final io.vertx.ext.web.handler.DigestAuthHandler delegate;
  
  public DigestAuthHandler(io.vertx.ext.web.handler.DigestAuthHandler delegate) {
    this.delegate = delegate;
  }

  public DigestAuthHandler(Object delegate) {
    this.delegate = (io.vertx.ext.web.handler.DigestAuthHandler)delegate;
  }

  public io.vertx.ext.web.handler.DigestAuthHandler getDelegate() {
    return delegate;
  }

  /**
   * Something has happened, so handle it.
   * @param event the event to handle
   */
  public void handle(io.vertx.rxjava.ext.web.RoutingContext event) { 
    delegate.handle(event.getDelegate());
  }

  /**
   * Create a digest auth handler
   * @param vertx the vertx instance
   * @param authProvider the auth provider to use
   * @return the auth handler
   */
  public static io.vertx.rxjava.ext.web.handler.DigestAuthHandler create(io.vertx.rxjava.core.Vertx vertx, io.vertx.rxjava.ext.auth.htdigest.HtdigestAuth authProvider) { 
    io.vertx.rxjava.ext.web.handler.DigestAuthHandler ret = io.vertx.rxjava.ext.web.handler.DigestAuthHandler.newInstance((io.vertx.ext.web.handler.DigestAuthHandler)io.vertx.ext.web.handler.DigestAuthHandler.create(vertx.getDelegate(), authProvider.getDelegate()));
    return ret;
  }

  /**
   * Create a digest auth handler, specifying the expire timeout for nonces.
   * @param vertx the vertx instance
   * @param authProvider the auth service to use
   * @param nonceExpireTimeout the nonce expire timeout in milliseconds.
   * @return the auth handler
   */
  public static io.vertx.rxjava.ext.web.handler.DigestAuthHandler create(io.vertx.rxjava.core.Vertx vertx, io.vertx.rxjava.ext.auth.htdigest.HtdigestAuth authProvider, long nonceExpireTimeout) { 
    io.vertx.rxjava.ext.web.handler.DigestAuthHandler ret = io.vertx.rxjava.ext.web.handler.DigestAuthHandler.newInstance((io.vertx.ext.web.handler.DigestAuthHandler)io.vertx.ext.web.handler.DigestAuthHandler.create(vertx.getDelegate(), authProvider.getDelegate(), nonceExpireTimeout));
    return ret;
  }

  /**
   * The default nonce expire timeout to use in milliseconds.
   */
  public static final long DEFAULT_NONCE_EXPIRE_TIMEOUT = io.vertx.ext.web.handler.DigestAuthHandler.DEFAULT_NONCE_EXPIRE_TIMEOUT;
  public static DigestAuthHandler newInstance(io.vertx.ext.web.handler.DigestAuthHandler arg) {
    return arg != null ? new DigestAuthHandler(arg) : null;
  }

}
